;; Copyright 2010, 2011, 2013 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; /usr/share/emacs/23.4/lisp/emacs-lisp/cl-macs.el.gz
;; /usr/share/emacs/24.3/lisp/emacs-lisp/cl-macs.el.gz
;; /so/emacs24.3/emacs24-24.3+1/lisp/emacs-lisp/cl-macs.el


;;-----------------------------------------------------------------------------
;; commentary examples

(add-to-list 'load-path (expand-file-name ".."))
(eval-when-compile
  (require 'cl)
  (require 'cl-loop-aplist))

(setq foo '(("red"   . 6)
            ("green" . 8)))
(loop for x being the alist-values of foo
                        sum x)


(loop for x being the plist-values of '("red" 6 "green" 8)
      sum x)

(loop for (x . y) in '((1 . 2) (3 . 4))
      collect y)

(loop for (x y) on '(1 2 3 4) by 'cddr
      collect x)

(setq foo '((1 . 123)
            (2 . 456)))
(pp (macroexpand '(loop for v being the alist-values of-ref foo
        using-ref (alist-keys k)
        sum (* v k))  )
    (current-buffer))

















;;-----------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))

;;-----------------------------------------------------------------------------
;; of-ref

(let ((my-list (list 100 200 300)))
  (loop for v in-ref my-list
        do
        (incf v))
  my-list)

(let ((my-list '((1 . 123)
                 (2 . 456))))
  (loop for v being the alist-values of-ref my-list
        using-ref (alist-keys k)
        do (incf v)
        do (decf k)
        finally return my-list))

(loop for x being the alist-values of '(("red"   . 6)
                                        ("green" . 8))
      sum x)



;;-----------------------------------------------------------------------------
;; hash table

(let ((table (make-hash-table :test 'equal)))
  (puthash 1 100 table)
  (puthash 2 200 table)
  (loop for x being the hash-values of-ref table
        do (incf x))
  table)


;;-----------------------------------------------------------------------------

(defconst foo '((100 . 1) (200 . 2)))
(put 'foo 'a 1)
(put 'foo 'b 2)

(loop for x being the alist-keys of foo
      collect x)
(loop for x being the alist-values of foo
      collect x)
(loop for x being the plist-keys of (symbol-plist 'foo)
      collect x)
(loop for x being the alist-keys of-ref foo
      do (incf x)
      finally return my-list)

(cl-prettyexpand '(let* ((sym 'foo)

       (width (loop for p being the plist-keys of (symbol-plist sym)
                    maximize (length (symbol-name p))))
       (fmt   (format "%%-%ds %%S\n" width)))
  (loop for p being the plist-keys of (symbol-plist sym)
        using (plist-values v)
        do (insert (format fmt p v)))))








(loop for x being the alist-keys of-ref foo
      do (incf x)
      finally return foo)
(loop for v being the alist-values of-ref foo
      do (incf v)
      finally return foo)

(pp (macroexpand '(loop for x being the alist-keys of foo
                        collect x))
    (current-buffer))



(cl-block-wrapper
 (catch '--cl-block-nil--
   (let*
       ((--cl-var-- foo)
	(x nil)
	(--cl-var-- nil))
     (while
	 (consp --cl-var--)
       (setq x
	     (car --cl-var--))
       (push x --cl-var--)
       (setq --cl-var--
	     (cdr --cl-var--)))
     (nreverse --cl-var--))))


(cl-prettyexpand '(loop for x being the plist-keys of (symbol-plist 'foo)
                        using (plist-values y)
                        do
                        (message "%S %S" x y)))

(block nil
  (let* ((--cl-var-- (symbol-plist 'foo))
         (x nil)
         (y nil))
    (while (consp --cl-var--)
      (setq x (car --cl-var--) y (cadr) --cl-var--)
      (message "%S %S" x y)
      (setq --cl-var-- (cddr --cl-var--)))
    nil))




(cl-prettyexpand '(loop for x being the elements of [a b c]
                        using (index y)
                        do
                        (message "%S %S" x y)))

(block nil
  (let* ((--cl-seq-- [a b c])
         (y 0)
         (x nil))
    (while (and --cl-seq-- (or (consp --cl-seq--) (< y (length --cl-seq--))))
      (setq x (if (consp --cl-seq--)
                  (car (prog1 --cl-seq-- (setq --cl-seq-- (cdr --cl-seq--))))
                (aref --cl-seq-- y)))
      (message "%S %S" x y)
      (setq y (1+ y)))
    nil))







;; (put 'alist-keys 'cl-loop-for-handler 'cl-loop-aplist)
;; (put 'alist-keys 'cl-loop-aplist-step     'cdr)
;; (put 'alist-keys 'cl-loop-aplist-accessor 'caar)
;; (put 'alist-keys 'cl-loop-aplist-other    'alist-values)
;; 
;; (put 'alist-values 'cl-loop-for-handler 'cl-loop-aplist)
;; (put 'alist-values 'cl-loop-aplist-step     'cdr)
;; (put 'alist-values 'cl-loop-aplist-accessor 'cdar)
;; (put 'alist-values 'cl-loop-aplist-other    'alist-keys)
;; 
;; (put 'plist-keys 'cl-loop-for-handler 'cl-loop-aplist)
;; (put 'plist-keys 'cl-loop-aplist-step     'cddr)
;; (put 'plist-keys 'cl-loop-aplist-accessor 'car)
;; (put 'plist-keys 'cl-loop-aplist-other    'plist-values)
;; 
;; (put 'plist-values 'cl-loop-for-handler 'cl-loop-aplist)
;; (put 'plist-values 'cl-loop-aplist-step     'cddr)
;; (put 'plist-values 'cl-loop-aplist-accessor 'cadr)
;; (put 'plist-values 'cl-loop-aplist-other    'plist-keys)






;; (put 'alist-keys   'cl-loop-for-handler 'cl-loop-aplist)
;; (put 'alist-values 'cl-loop-for-handler 'cl-loop-aplist)
;; (put 'plist-keys   'cl-loop-for-handler 'cl-loop-aplist)
;; (put 'plist-values 'cl-loop-for-handler 'cl-loop-aplist)
;; 
;; (defconst cl-loop-aplist-step '((alist-keys   . cdr)
;;                                 (alist-values . cdr)
;;                                 (plist-keys   . cddr)
;;                                 (plist-values . cddr)))
;; (defconst cl-loop-aplist-accessor '((alist-keys   . caar)
;;                                     (alist-values . cdar)
;;                                     (plist-keys   . car)
;;                                     (plist-values . cadr)))
;; (defconst cl-loop-aplist-other '((alist-keys   . alist-values)
;;                                  (alist-values . alist-keys)
;;                                  (plist-keys   . plist-values)
;;                                  (plist-values . plist-keys)))
;; 
;; (defun cl-loop-aplist (var)
;;   (or (eq 'of (car args))
;;       (error "Expected `of'"))
;; 
;;   (let ((temp (make-symbol "--cl-var--")))
;;     ;; `consp' loop test same as `in LIST'
;;     (push (list temp   (cl-pop2 args))  loop-for-bindings)
;;     (push (list 'consp temp)            loop-body)
;;     (push (list temp   (list (assoc-default word cl-loop-aplist-step) temp))
;;           loop-for-steps)
;; 
;;     (push (list var   nil)               loop-for-bindings)
;;     (push (list var   (list (assoc-default word cl-loop-aplist-accessor) temp))
;;           loop-for-sets)
;; 
;;     (when (eq (car args) 'using)
;;       (let* ((form  (cl-pop2 args))
;;              (other (first form))
;;              (var2  (second form)))
;;         (or (and (= (length form) 2)
;;                  (eq other (assoc-default word cl-loop-aplist-other)))
;;             (error "Bad `using' clause"))
;;         (push (list var2 nil)             loop-for-bindings)
;;         (push (list var2 (list (assoc-default other cl-loop-aplist-accessor)) temp)
;;               loop-for-sets)))))))



(put 'foo 'prop1 123)
(put 'foo 'bar   99999)
(put 'foo 'quux  "hello")

(let* ((sym 'foo)

       (width (loop for p being the plist-keys of (symbol-plist sym)
                    maximize (length (symbol-name p))))
       (fmt   (concat "%-" (number-to-string width) "s %S\n")))
  (loop for p being the plist-keys of (symbol-plist sym)
        using (plist-values v)
        do (insert (format fmt p v))))
prop1 123
bar   99999
quux  1



