;; Copyright 2011, 2012, 2013, 2014 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.



;;-----------------
;; wrapping conditional

(unless nil
  (defface my-test-face
    '((t
       (:foreground "orange")))
    "This is a face."))

(if t
    (defun foo () "True")
  (defun bar () "False"))


;;-----------------
;; /usr/share/emacs/site-lisp/apel/static.el

(require 'static)
(static-defconst my-test-stconst 123
  "This is a variable.")
(defun my-test-stconst ()
  "This is a function.")


;;-----------------
;; help buffer

(load "~/lisp/elisp-docstring-preview.el")
(progn
  (describe-function 'car)
  (elisp-docstring-preview--help-buffer 'describe-function 'car))
(progn
  (describe-variable 'emacs-version)
  (elisp-docstring-preview--help-buffer 'describe-variable 'emacs-version))
(progn
  (describe-face 'italic)
  (elisp-docstring-preview--help-buffer 'describe-face 'italic))
(progn
  (describe-character-set 'iso-8859-1)
  (elisp-docstring-preview--help-buffer 'describe-character-set 'iso-8859-1))
(progn
  (describe-coding-system 'euc-jp)
  (elisp-docstring-preview--help-buffer 'describe-coding-system 'iso-8859-1))
(progn
  (describe-font 'courier)
  (elisp-docstring-preview--help-buffer 'describe-font 'courier))

;;-----------------
(debug-on-entry 'custom-declare-face)
(cancel-debug-on-entry 'custom-declare-face)

(defvar my-test 123
  "This xx is a variable docstring")
(defvar-maybe my-test 123
  "This is a variable docstring")

(defun my-test ()
  "This is a function docstring"
  123)
(defun* my-test ()
  "This is a another function docstring"
  123)

(defalias 'my-test 'identity
  "This is an alias docstring") ;; docstring new in emacs21
(defalias 'my-test 'identity)

(defvaralias 'my-test 'emacs-version
  "This is a variable alias docstring")

(defface my-test '((t (:foreground "blue")))
  "This fjdsk is a blue face docstring.")

(defface my-test '((t (:foreground "red")))
  "This is a red face docstring.")

(progn
  (require 'calc)
  (defmath myfact (n)
    "my docstring"
    (if (> n 0)
        (* n (myfact (1- n)))
      1)))

(define-modify-macro my-test (&optional (n 1)) +
  "Docstring of modify macro")

(define-ibuffer-column my-test (:name "Server")
  "column docstring"
  123)
(define-ibuffer-sorter my-test
  "My sorter."
  (:description "mine")
  t)
(define-ibuffer-filter my-test2
  "My test ..."
  (:description "some thing"
   :reader (read-from-minibuffer "Filter by my test: "))
  123)

(define-ibuffer-op ibuffer-do-my-test (command)
  "Run ..."
  (:interactive "sSomething: "
   :opstring "My test op"
   :modifier-p nil)
  123)
(define-ibuffer-op my-test (command)
  "Run something ..."
  (:interactive "sSomething: "
   :opstring "My test op"
   :modifier-p nil)
  123)


(require 'eieio)
(defclass my-baseclass ()
  ((slot-A :initarg :slot-A)
   (slot-B :initarg :slot-B))
  "My Baseclass.")

(define-widget 'my-test 'item
  "Control something ..."
  :format "%[[%t]%]"
  :action 'identity)

(describe-function (lambda (x)
                     "my docstring"
                     (+ x 123)))

(describe-function 'jdks)
(defmacro defunknown (&rest args)
  '(progn))
(defunknown my-jfdks ()
  123)
(put 'defunknown
     'elisp-docstring-preview-name-prefix "myblah-")

(put 'my-test 'my-property-name 123)


;;-----------------
(let (ret
      ;; (x (make-symbol "foo's"))
      (bar\'s 123))
  (mapatoms (lambda (symbol)
              (when (string-match "'" (symbol-name symbol))
                  (push symbol ret)
                  (message "%S" symbol))))
  ret)

(progn
  (defvar foo\'s 123)
  (describe-variable 'foo\'s)
  (elisp-docstring-preview--help-symbol-p 'foo\'s))

;;-----------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))


;;-----------------
;; window-start

(defmacro perl-pod-preview--save-wpoints (&rest body)
  "Save `point', `window-point' and `window-start' positions as line/column.
The use of line/column means BODY can change line contents and
the displayed parts of the current buffer are unchanged."

  ;; (declare (debug t))  ;; emacs22,xemacs21, or 'cl
  `(let ((buffer  (current-buffer))
         (wpoints (perl-pod-preview--wpoints-get (current-buffer))))
     ,@body
     (perl-pod-preview--wpoints-set wpoints buffer)))




(elisp-docstring-preview--goto-linecol '(2 . 5))
(elisp-docstring-preview--wpoints-get (current-buffer))
(elisp-docstring-preview--linecol (point))
(let ((wpoints (elisp-docstring-preview--wpoints-get (current-buffer))))
  (elisp-docstring-preview--wpoints-set (current-buffer) wpoints)
  wpoints)





(loop for x from 1 to 3
      for y from 10 to 12
      collect (cons x y))

(let ((buffer (current-buffer)))
  (cons (elisp-docstring-preview--linecol (point))
        (loop for frame being the frames
              append
              (loop for window being the windows of frame
                    if (eq buffer (window-buffer window))
                    collect
                    (list window
                          (elisp-docstring-preview--linecol (window-start window))
                          (elisp-docstring-preview--linecol (window-point window)))))))



;;----------------
;; junk

(defun elisp-docstring-preview--help-symbol-p (symbol)
  "Return non-nil if the *Help* buffer is currently describing SYMBOL.
This is done by looking at the first word in the buffer."
  (and (get-buffer "*Help*")
       (with-current-buffer "*Help*"
         (save-excursion
           (goto-char (point-min))
           (and (looking-at "[^ \t]+")
                (let ((str (match-string 0)))
                  (if (string-match str "'s\\'")
                      (setq str (substring str 0 -2)))
                  (equal str (symbol-name symbol))))))))

(defun elisp-docstring-preview--help-symbol-name ()
  "Return a string which is the symbol the *Help* buffer is describing.
If there's no *Help* buffer or its contents are unrecognised then
return nil.

This is done by looking at the *Help* buffer contents.  It might
be better to check the top of `help-xref-stack'.

Any trailing \"'s\" from `describe-variable' is removed, and any
backslashing of unusual characters is undone, so the returned
string should equal `symbol-name' of whatever is described."

  (and (get-buffer "*Help*")
       (with-current-buffer "*Help*"
         (if (boundp 'help-xref-stack-item)
             (symbol-name (cadr help-xref-stack-item))

           (save-excursion
             (goto-char (point-min))
             (and (looking-at "[^ \t]+")
                  (let ((str (match-string 0)))
                    (setq str (elisp-docstring-preview--replace-regexp-in-string
                               "\\\\\\(.\\)" "\\1" str t))
                    (setq str (elisp-docstring-preview--replace-regexp-in-string
                               "'s\\'" "" str t t))
                    (setq str (elisp-docstring-preview--replace-regexp-in-string
                               "\\``\\(.*\\)'\\'" "\\1" str t))
                    str)))))))

;;------------------------------------------------------------------------------
;; maybe ...

(defun elisp-docstring-form-at-point ()
  "Return the form at point, possibly descending into conditionals.
This is an internal part of elisp-docstring-preview.el.

If point is in a conditional form like

    (when something
      (defun foo () 123))

then the return is the (defun ...) part.  For an `if' like

    (if something
        (defun foo () 123)
      (defun bar () 123))

The return is whichever leg contains point.

`static-if', `static-when' and `static-unless' from APEL
static.el are recognised too."

  (save-excursion
    (let ((orig-point (point))
          (more       t)
          ret)
      (end-of-defun)
      (beginning-of-defun)
      (while more
        (setq ret (save-excursion (read (current-buffer))))
        (cond ((and (memq (car-safe ret)
                          '(when unless static-when static-unless))
                    (cdr ret))
               (down-list)
               (forward-sexp 2)) ;; past `when' and condition

              ((and (memq (car-safe ret) '(if static-if))
                    (cdr ret))
               (down-list)
               (forward-sexp 2) ;; past `if' and condition
               (let ((pos (save-excursion (forward-sexp) (point))))
                 (if (< pos orig-point) ;; orig-point is in "false" leg
                     (goto-char pos))))

              (t
               (setq more nil))))
      ret)))

;;------------------------------------------------------------------------------
;;; d-emacs-lisp-docstring-preview.el ends here

