;; Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2019 Kevin Ryde.        -*- coding: utf-8 -*-

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;;-----------------------------------------------------------------------------
;; man-completion-at-point-regexp-filename

(progn
  (string-match man-completion-at-point-regexp-filename "/bin/sh\n\n\n")
  (match-end 0))


;;-----------------------------------------------------------------------------
;; entries should match Man-name-regexp

(remove* (concat "\\`" Man-name-regexp "\\((.*)\\)?\\'"
                 "\\| ")
         (man-completion-cache)
         :test 'string-match
         :key 'car)

;; entries which are all numbers
(remove* (concat "\\`" "[0-9]+" "\\((.*)\\)?\\'")
         (remove* " " (man-completion-cache) :test 'string-match :key 'car)
         :test-not 'string-match
         :key 'car)

(compile "find /usr/share/man -type f | xargs zgrep -E '\\b[0-9]+[(]'")
(compile "find /usr/share/man -type f | xargs zcat | grep -E '\\b[0-9]+[(]'")

;;-----------------------------------------------------------------------------
;; entries with "/"

(remove* "/"
         (man-completion-cache)
         :test-not 'string-match
         :key 'car)

;;--------------------------------------

(setq Man-name-regexp "[-a-zA-Z0-9_­+][-a-zA-Z0-9_.:­+]*")

(progn
  (setq Man-name-regexp (concat "[-a-zA-Z0-9_"
                                "­"
                                "‐"
                                "+][-a-zA-Z0-9_.:"
                                "­"
                                "‐"
                                "+]*"))
  (require 'man))

Man-hyphenated-reference-regexp

;;-----------------------------------------------------------------------------
;; /etc files with man pages

(progn
  (require 'man-completion)
  (require 'cl)
  (let ((manpages (man-completion-cache t))
        filenames)
    (setq filenames (directory-files "/etc"
                                     t   ;; full
                                     nil ;; match
                                     t)) ;; nosort

    ;; one level down
    (let ((dirs (remove-if-not 'file-directory-p filenames)))
      (setq filenames nil)
      (dolist (dir dirs)
        (setq filenames (append filenames
                                (directory-files dir
                                                 t ;; not full
                                                 nil ;; match
                                                 t))))) ;; nosort

    ;; no directories
    (setq filenames (remove-if 'file-directory-p filenames))

    (message "intersection %d %d" (length filenames) (length manpages))
    (redisplay)

    (let ((filenames-hash (make-hash-table :test 'equal))
          (manpages-hash  (make-hash-table :test 'equal)))
      (dolist (filename filenames)
        (puthash (file-name-nondirectory filename) filename filenames-hash))
      (dolist (pair manpages)
        (puthash (car pair) (cdr pair) manpages-hash))
      '(setq manpages (remove-if-not (lambda (pair)
                                       (gethash (car pair) hash))
                                     manpages))

      (setq manpages (remove-if-not (lambda (filename)
                                      (gethash (file-name-nondirectory filename)
                                               manpages-hash))
                                    filenames)))

    (message "insert")
    (switch-to-buffer "xx")
    (erase-buffer)
    (dolist (pair manpages)
      (insert (format "%S\n" pair)))
    (goto-char (point-min))))

;;-----------------------------------------------------------------------------

(completing-read "entry: " '(("xx-aa" . "123")
                             ("yy-aa" . "456")))

(completing-read "entry: " '(("Foo" . "123")
                             ("foo" . "456")))


;;-----------------------------------------------------------------------------
;; use sort 'stable'; default to sort(3perl)

(defun foo ()
  (and (save-excursion
         (re-search-backward "\\(['\"]\\|^[ \t]*\\(#[ \t]*\\)?\\)use[ \t]*\\="
                             (line-beginning-position) t))
       (looking-at "[a-zA-Z_][a-zA-Z0-9_]*[ \t]*\\(;[ \t]*\\(#.*\\)?$\\|[ \t]\\(['\"]\\|qw\\|[a-zA-Z_][a-zA-Z0-9_]*[ \t\r\n]+=>\\)\\)")))
use sort; # default to sort(3perl)
# use sort qw();
use sort 'stable'; # default to sort(3perl)
use constant FOO => 1;


;;-----------------------------------------------------------------------------

(progn
  ;; (add-to-list 'load-path (expand-file-name ".."))
  (add-to-list 'load-path (expand-file-name "~/lisp"))
  (require 'man-completion))

(setq completion-pcm-word-delimiters "-_. :")
(setq completion-pcm-word-delimiters ":")
(completion-pcm--prepare-delim-re completion-pcm-word-delimiters)

(completing-read "foo: " '(("foo:bar" . "123")))
(completing-read "foo: " '(("aaa-bar" . "123")))


(completing-read "foo: " '(("foo" . "123")
                           ("bar" . "456")))
completing-help-groups

(progn
  (or (member 'utf-8 (coding-system-list))
      (require 'un-define))
  (find-file "d-man-completion.utf8"))

(define-key global-map [?\C-h ?\C-m] 'man)
cat
chmod
Client::DNS
DNS
PoCo
PoCo::JobQueue
français
sort.pm
-MO=Concise
-MFile::Spec
/bin/cat
/usr/bin/cat
/usr/local/bin/cat
/sbin/cat
/usr/sbin/cat
/usr/local/sbin/cat
/opt/package/sbin/cat
L<perlvar/ENV>
L<perlvar/"ENV">
L<perlvar>
ca-
t

(man-completion-transform-perl "AptPkg::Cache(3pm)")

(setq Man-switches nil)

(add-to-list 'man-completion-at-point-functions
             'man-completion-transform-perl)
(add-to-list 'man-completion-at-point-functions
             'man-completion-transform-poco)
man-completion-at-point-regexp

(let ((str "AptPkg::Cache(3pm)"))
  (assoc (man-completion-name-sans-section str)
         (man-completion-cache)))

(add-text-properties 0 (length name)
                     '(help-echo "some text")
                     name)

;;--------------------------------------
;; man-completion-hook-to-list

(add-hook 'my-hook 'identity)
(progn
  (switch-to-buffer "x")
  (make-local-variable 'my-hook)
  (add-hook 'my-hook 'xxx t t)
  my-hook)

;;--------------------------------------
;; .pm to section

sort.pm
(remove-if (lambda (elem)
             (not (string-match "\\`sort" (car elem))))
           (man-completion-cache))

(assoc* (concat "\\`"
                (regexp-quote "sort")
                "(3p\\(m\\|perl\\))\\'")
        (man-completion-cache)
        :test 'string-match)

(assoc* "\\`sort("
        (man-completion-cache)
        :test 'string-match)

(progn
  (require 'cl)
  (let ((pages (mapcar 'car (man-completion-cache)))
        (print-level nil)
        ret)
    (setq pages (remove* "(3pm)\\'\\|(3perl)\\'" pages
                         :test-not 'string-match))
    (setq pages (remove* "::" pages
                         :test 'string-match))
    (dolist (str pages)
      (string-match "\\(.*\\)(" str)
      (let* ((base (match-string 1 str))
             (elem (assoc base ret)))
        (unless elem
          (setq elem (list base))
          (push elem ret))
        (push str (cdr elem))))

    (setq ret (remove* 1 ret :key (lambda (elem) (length (cdr elem)))))
    (switch-to-buffer "x")
    (pp ret (get-buffer "x"))))

;;--------------------------------------
;; boundaries

((eq (car-safe action) 'boundaries) ;; emacs23
                  ;; boundaries at "/" like `completion-file-name-table', but
                  ;; skipping any "-l " at the start of STR
                  (let* ((suffix (cdr action))
                         (start  (+ (length before)
                                    (length (file-name-directory fname))))
                         (end    (string-match "/" suffix)))
                    (cons 'boundaries (cons start end))))
                 
;;--------------------------------------
;; byte-compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))

(progn
  (elint-initialize)
  (find-file "../man-completion.el")
  (elint-current-buffer))

;;--------------------------------------


(let ((elfile (expand-file-name "../man-completion.el"))
      (tempfile "/tmp/x.el")
      (print-level nil))
  (require 'autoload)
  (find-file tempfile)
  (erase-buffer)
  (generate-file-autoloads elfile)
  (save-buffer)
  (byte-compile-file tempfile)
  (delete-file (concat tempfile "c"))
  (delete-file tempfile))

;;--------------------------------------

(let ((count 0))
  (add-to-list 'load-path "..")
  (require 'man-completion)

  (let ((default-directory "/no/such/dir"))
    (man-completion-reset)
    (man-completion-cache)
    (or (equal default-directory "/no/such/dir")
        (error "oops, default-directory changed"))
    (setq count (1+ count)))

  (dolist (elem '(("AptPkg::Cache(3pm)" "AptPkg::Cache")
                  ("cat" nil)
                  ("cat::fjsdkfjdks" "cat")
                  ("fjksd::jfdksl::fjdks" nil)
                  ("Symbol::Xyzzy" "Symbol")))
    (let* ((str  (nth 0 elem))
           (want (nth 1 elem))
           (got  (man-completion-perl-prune-suffix str)))
      (unless (equal want got)
        (error "prune-suffix str %S got %S want %S\n" str got want))
      (setq count (1+ count))))

  (dolist (elem '(("cat(1)" "cat")
                  ("cat(1x)" "cat")
                  ("1 cat" "cat")
                  ("3perl cat" "cat")
                  ("822-date" "822-date")
                  ("822-date(1)" "822-date")))
    (let* ((str  (nth 0 elem))
           (want (nth 1 elem))
           (got  (man-completion-name-sans-section str)))
      (unless (equal want got)
        (error "sans-section str %S got %S want %S\n" str got want))
      (setq count (1+ count))))

  (dolist (elem '(("PoCo" "POE::Component")
                  ("PoCo::JobQueue" "POE::Component::JobQueue")
                  ("Foo" nil)
                  ("zPoCoZ" nil)))
    (let* ((str  (nth 0 elem))
           (want (nth 1 elem))
           (got  (man-completion-transform-poco str)))
      (unless (equal want got)
        (error "transform-poco str %S got %S want %S\n" str got want))
      (setq count (1+ count))))

  (dolist (elem '(("DNS" nil)
                  ("ProhibitHardTabs" "Perl::Critic::Policy::CodeLayout::ProhibitHardTabs")
                  ("Client::DNS" "POE::Component::Client::DNS")))
    (let* ((str  (nth 0 elem))
           (want (nth 1 elem))
           (got  (man-completion-perl-expand-prefix str)))
      (unless (equal want got)
        (error "expand-prefix str %S got %S want %S\n" str got want))
      (setq count (1+ count))))

  ;; cat
  ;; chmod

  (let ((lst (man-completion-cache)))
    (dolist (entry lst)
      (unless (and (consp entry)
                   (stringp (car entry))
                   (eq nil (cdr entry)))
        (error "cache bad entry %S" entry))))

  (dolist (elem '(("File::Spec"            "File::Spec")
                  ("man-\n path(5)"        "manpath(5)")
		  ("man­\n path(5)"        "manpath(5)") ;; latin-1
		  ;; ("+----Glib::Foo"        "Glib::Foo")
		  ("["      "[")
		  ("[foo]"  "foo" 1)
		  ("[-foo]" "foo" 2)
                  ))
    (let* ((str    (nth 0 elem))
           (want   (nth 1 elem))
           (offset (or (nth 2 elem) 0))
           (got    (with-temp-buffer
                     (insert str)
                     (goto-char (+ offset (point-min)))
                     (man-completion-at-point))))
      (unless (string-equal want got)
        (error "at-point str %S got %S want %S\n" str got want)))
    (setq count (1+ count)))

  count)




;; (man-completion-read)
;;  ch-
;;  mod



;;   ch-
;;   own

;; (put 'manpage 'thing-at-point 'man-completion-at-point)

;; (put 'manpage 'bounds-of-thing-at-point 'my-func)

;; (cons (match-beginning 1) (match-end 1))
;;    (and (thing-at-point-looking-at "\
;; [^ \t\r\n\f():;,-]\\([^ \t\r\n\f():;,]*[^ \t\r\n\f():;,-]\\)?\
;; \\(::[^ \t\r\n\f():;,-]\\([^ \t\r\n\f():;,]*[^ \t\r\n\f():;,-]\\)?\\)*\
;; \\(([0-9][A-Za-z0-9_]*)\\)?")
;;              (cons (match-beginning 0) (match-end 0)))))

;; (let ((allowed "A-Za-z0-9+-_."


;; (thing-at-point 'manpage)
;; FD_SET
;; cat(2)
;; cat()-1

;; (defun man-completion-read


;; (ad-disable-advice 'man 'before 'my-woman-prompt)




;; ;; Foo::Bar->

;; (progn
;;   (require 'thingatpt)
;;   (switch-to-buffer "*man-db-completion temp*")
;;   (setq buffer-undo-list t)
;;   (shell-command "accessdb" (current-buffer))
;;   (while (re-search-forward "^\\(.*?\\)\\( ->.*\\)$" nil t)
;;     (replace-match "" t t nil 2))

;;   (goto-char (point-min))
;;   (while (re-search-forward "~.*$" nil t)
;;     (replace-match "" t t)))

;; (progn
;;   (switch-to-buffer "*man-db-completion temp*")
;;   (goto-char (point-min))
;;   (while (not (eobp))
;;     (let ((got  (my-func))
;;           (want (cons (point-at-bol) (line-end-position))))
;;       (if (equal got want)
;;           (delete-region (point-at-bol) (1+ (line-end-position)))
;;         (message "%s" (buffer-substring (point-at-bol) (1+ (line-end-position))))
;;         (forward-line)))))





;;   (while (re-search-forward " +-.*$" nil t)
;;     (replace-match "" t t nil))
;;   (goto-char (point-min))
;;   (while (re-search-forward "~.*$" nil t)
;;     (replace-match "" t t)))


(progn
  (setq locale-coding-system 'utf-8)
  (setq Man-switches "-Tutf8")
  (man "chmod"))


(let ((str "-l x"))
  (if (string-match "^\\(-l \\)\\(.*\\)\\|[./]" str)
      (let* ((before (or (match-string 1 str) ""))
             (fname  (or (match-string 2 str) str))
             (dir    (file-name-directory fname))
             (part   (file-name-nondirectory fname)))
        fname)))

;; _exit
;; exit.

(length (man-completion-cache))
(length (file-expand-wildcards "/usr/share/man/man*/*"))
(with-temp-buffer
  (let ((process-connection-type nil)) ;; pipe
    (call-process "man" nil t nil "-k" "")
    (count-lines (point-min) (point-max))))

(progn
  (setq files nil)
  (dolist (filename (file-expand-wildcards "/usr/share/man/man*/*"))
    (or (string-match "/\\([^/]*?\\)\\(\\.[0-9][A-Za-z0-9]*\\)\\(\\.gz\\)?\\'" filename)
        (error "oops: %S" filename))
    (let ((str (match-string 1 filename)))
      (setq str (intern str))
      (setq files (cons str files))))
  (setq files (sort files 'string<))
  (length files))
files
    
(progn
  (setq man-k nil)
  (with-temp-buffer
    (let ((process-connection-type nil)) ;; pipe
      (call-process "man" nil t nil "-k" ""))
    (goto-char (point-min))
    (while (not (eobp))
      (or (looking-at "\\(.*?\\) (\\([^)]+\\))")
          (error "oops"))
      (let ((str (concat (match-string 1) ;; "." (match-string 2)
                         )))
        (setq str (intern str))
        (setq man-k (cons str man-k)))
      (forward-line)))
  (setq man-k (sort man-k 'string<))
  (length man-k))
man-k

(length man-k)
(length files)
(require 'cl)
(setq d (set-difference man-k files))
(length d)
(setq e (set-difference files man-k))
(length e)

(progn (defalias 'foobar (lambda () 123)) (foobar))



;;------------------------------------------------------------
;; xemacs


(open-database "/var/cache/man/index.db" "dbm" nil "r")

PoCo

;;-----------------------------------------------------------------------------
;; declare-function

;; (require 'cl)
;; (labels ((declare-function () nil))
;;   (if (fboundp 'declare-function)
;;       (declare-function foo "foo.el")))

(eval '(declare-function foo "foo.el"))

;; (eval-when-compile (eval '(declare-function foo "foo.el")))
;; 
;; (eval-when-compile (eval '(declare-function foo "foo.el")))

(defun bar ()
  (foo))
;; (byte-compile-file "/tmp/x.el")
;; (symbol-plist 'declare-function)


;;--------------------------------------
;; unused

(defmacro man-completion-with-temp-filename (&rest body)
  "Create a `filename' for use by the BODY forms.
An `unwind-protect' removes the file no matter what BODY does."
  ;; (declare (debug t))  ;; emacs22,xemacs21, or 'cl
  `(let ((filename (man-completion--make-temp-file "man-completion-")))
     (unwind-protect
         (progn ,@body)
       (delete-file filename))))

    (man-completion-with-temp-filename
     (let ((process-environment (copy-sequence process-environment))
           (default-directory "/")) ;; in case inherited doesn't exist)
       (setenv "COLUMNS" "999")
       (call-process shell-file-name
                     nil  ;; stdin
                     nil  ;; output
                     nil  ;; no display
                     shell-command-switch
                     (concat manual-program " -k \"\" >" filename)))


        ;; beginning-of-line to match start is the page name, the first
        ;; parens "(foo)" is the section, then skip "[foo]" real page and "
        ;; - " to get to the description part.
        ;;
        ;; The loop runs a touch faster with the name part picked out by
        ;; `line-beginning-position' rather than a "^\\(.*\\)" in the
        ;; regexp.
        ;;
        (goto-char (point-min))
        (while (re-search-forward "\
 (\\([^)]+?\\))\
\\( \\[[^]*]\\)?\
\\( +- \\)?\
\\(.*\\)" nil t)
          ;; one \\( \\) group per line
          (let ((name (buffer-substring (line-beginning-position)
                                        (match-beginning 0)))
                (sect (match-string 1))
                (desc (and want-desc (match-string 4))))


;;--------------------------------------
;; pipe speed

(let ((t1 (float-time)))
  (with-temp-buffer
    (call-process shell-file-name
                  nil  ;; stdin
                  nil  ;; stdout+err
                  nil  ;; no display
                  shell-command-switch
                  "cat /usr/share/emacs/23.2/etc/DOC-23.2.1 >/tmp/foo")
    (insert-file-contents "/tmp/foo"))
  (- (float-time) t1))

(let ((t1 (float-time)))
  (with-temp-buffer
    (call-process shell-file-name
                  nil  ;; stdin
                  t    ;; stdout+err
                  nil  ;; no display
                  shell-command-switch
                  "cat /usr/share/emacs/23.2/etc/DOC-23.2.1"))
  (- (float-time) t1))


;; crunching a buffer read in one block is a touch quicker, or maybe only
;; defers gcs

(let ((t1 (float-time)))
  (dotimes (i 10)
    (with-temp-buffer
      (call-process shell-file-name
                    nil  ;; stdin
                    nil  ;; stdout+err
                    nil  ;; no display
                    shell-command-switch
                    "man -k '' >/tmp/foo")
      (insert-file-contents "/tmp/foo")
      (my-parse)
      (setq size (- (point-max) (point-min)))))
  (format "%s %s" size (- (float-time) t1)))

(let ((t1 (float-time))
      size)
  (dotimes (i 10)
    (with-temp-buffer
      (call-process shell-file-name
                    nil  ;; stdin
                    t    ;; stdout+err
                    nil  ;; no display
                    shell-command-switch
                    "man -k ''")
      (my-parse)
      (setq size (- (point-max) (point-min)))))
  (format "%s %s" size (- (float-time) t1)))


(byte-compile 'my-parse)
(defun my-parse (&optional want-desc)
  (goto-char (point-min))
  (let (ret prev prevsect prevdesc)
    (while (re-search-forward "\
\\(.*\\) (\\([^)]+\\))\
\\( \\[[^]*]\\)?\
\\( +- \\)?\
\\(.*\\)" nil t)
      ;; (buffer-substring (point-at-bol)
      ;;                                       (- (match-beginning 1) 2))
      ;; one \\( \\) group per line
      (let ((name (match-string 1))
            (sect (match-string 2))
            (desc (and want-desc (match-string 5))))

        ;; If previous name was the same then put it with its prevsect
        ;; like "chmod(1)" and also the present one like "chmod(2)".
        (if (equal name prev)
            (progn
              (setq ret (cons (cons (concat name "(" sect ")")
                                    desc)
                              ret))
              (when prevsect
                (setq ret (cons (cons (concat prev "(" prevsect ")")
                                      prevdesc)
                                ret))
                (setq prevsect nil)))

          ;; If previous name different then just a solitary entry for
          ;; this one, like "chmod".
          (setq ret      (cons (cons name desc) ret)
                prev     name
                prevsect sect
                prevdesc desc))

        ;; always an entry like "1 chmod" or "2 chmod" with the sect
        (setq ret (cons (cons (concat sect " " name) desc)
                        ret))))))


;;--------------------------------------
;; load/save cache

(let ((print-level  nil)
      (print-length nil))
  (with-temp-file "/tmp/x.el"
    (man-completion-index-files-changed-p)
    (insert (pp-to-string man-completion-index-files-seen))
    (insert "(\n")
    (dolist (elem (man-completion-cache t))
      (insert (prin1-to-string elem) "\n"))
    (insert ")\n")))

(let ((t1 (float-time)))
  (with-temp-buffer
    (insert-file-contents "/tmp/x.el")
    (goto-char (point-min))
    (read (current-buffer))
    (my-collapse (setq man-completion-cache
                       (read (current-buffer)))))
  (- (float-time) t1))

(progn
  (find-file "/tmp/x"))

(defun my-collapse (alist)
  (let (prev)
    (dolist (elem alist)
      (if (equal (cdr elem) prev)
          (setcdr elem prev)
        (setq prev (cdr elem))))))
(byte-compile 'my-collapse)


(progn
  (require 'man-completion)
  (byte-compile 'man-completion-cache)
  (man-completion-reset)
  (let ((t1 (float-time)))
    (man-completion-cache)
    (format "%S  %S"
            (- (float-time) t1)
            (length man-completion-cache))))


;; .4 without name .*, .5 with

(man-completion-reset)
(length (man-completion-cache))

(defun man-completion-cache (&optional want-desc)

  (with-temp-buffer

            (let ((default-directory "/")) ;; in case inherited doesn't exist
              (let ((process-environment (copy-sequence process-environment))
                    (process-connection-type nil))
                (setenv "COLUMNS" "999")
                (call-process shell-file-name
                              nil
                              nil
                              nil
                              shell-command-switch
                              (concat manual-program " -k \"\" >/tmp/x"))))
            (insert-file-contents "/tmp/x")

;;     (setq default-directory "/") ;; in case inherited doesn't exist
;;     (let ((process-environment (copy-sequence process-environment))
;;           (process-connection-type nil))
;;       (setenv "COLUMNS" "999")
;;       (call-process shell-file-name
;;                     nil
;;                     t
;;                     nil
;;                     shell-command-switch
;;                     (concat manual-program " -k \"\"")))

    (let (ret prev prevsect prevdesc)

      ;; "man -k" gives lines like
      ;;     cat (1)              - concatenate files and print ...
      ;; or for an "alias"
      ;;     boot-scripts (7) [boot] - General description of boot sequence
      ;;
      ;; ".*?" is the page name, the first parens "(foo)" is the section,
      ;; then skip "[foo]" real page and " - " to get to the description
      ;; part.
      ;;
      (goto-char (point-min))
      (while (re-search-forward "\
\\(.*\\) (\\([^)]+\\))\
\\( \\[[^]*]\\)?\
\\( +- \\)?\
\\(.*\\)" nil t)
        ;; (buffer-substring (point-at-bol)
        ;;                                       (- (match-beginning 1) 2))
        ;; one \\( \\) group per line
        (let ((name (match-string 1))
              (sect (match-string 2))
              (desc (and want-desc (match-string 5))))

          ;; If previous name was the same then put it with its prevsect
          ;; like "chmod(1)" and also the present one like "chmod(2)".
          (if (equal name prev)
              (progn
                (setq ret (cons (cons (concat name "(" sect ")")
                                      desc)
                                ret))
                (when prevsect
                  (setq ret (cons (cons (concat prev "(" prevsect ")")
                                        prevdesc)
                                  ret))
                  (setq prevsect nil)))

            ;; If previous name different then just a solitary entry for
            ;; this one, like "chmod".
            (setq ret      (cons (cons name desc) ret)
                  prev     name
                  prevsect sect
                  prevdesc desc))

          ;; always an entry like "1 chmod" or "2 chmod" with the sect
          (setq ret (cons (cons (concat sect " " name) desc)
                          ret))))

      (setq man-completion-cache (nreverse ret))))))










(defun man-completion-cache (&optional want-desc)

  (with-temp-buffer

    ;;     (let ((default-directory "/")) ;; in case inherited doesn't exist
    ;;       (let ((process-environment (copy-sequence process-environment))
    ;;             (process-connection-type nil))
    ;;         (setenv "COLUMNS" "999")
    ;;         (call-process shell-file-name
    ;;                       nil
    ;;                       nil
    ;;                       nil
    ;;                       shell-command-switch
    ;;                       (concat manual-program " -k \"\" >/tmp/x"))))
    ;;     (insert-file-contents "/tmp/x")

    (setq default-directory "/") ;; in case inherited doesn't exist
    (let ((process-environment (copy-sequence process-environment))
          (process-connection-type nil))
      (setenv "COLUMNS" "999")
      (call-process shell-file-name
                    nil
                    t
                    nil
                    shell-command-switch
                    (concat manual-program " -k \"\"")))

    (let (ret prev prevsect prevdesc)

      ;; "man -k" gives lines like
      ;;     cat (1)              - concatenate files and print ...
      ;; or for an "alias"
      ;;     boot-scripts (7) [boot] - General description of boot sequence
      ;;
      ;; ".*?" is the page name, the first parens "(foo)" is the section,
      ;; then skip "[foo]" real page and " - " to get to the description
      ;; part.
      ;;
      (goto-char (point-min))
      (while (re-search-forward "\
 (\\([^)]+\\))\
\\( \\[[^]*]\\)?\
\\( +- \\)?\
\\(.*\\)" nil t)

        ;; one \\( \\) group per line
        (let ((name (buffer-substring (point-at-bol)
                                      (- (match-beginning 1) 2)))
              (sect (match-string 1))
              (desc (and want-desc (match-string 4))))

          ;; If previous name was the same then put it with its prevsect
          ;; like "chmod(1)" and also the present one like "chmod(2)".
          (if (equal name prev)
              (progn
                (setq ret (cons (cons (concat name "(" sect ")")
                                      desc)
                                ret))
                (when prevsect
                  (setq ret (cons (cons (concat prev "(" prevsect ")")
                                        prevdesc)
                                  ret))
                  (setq prevsect nil)))

            ;; If previous name different then just a solitary entry for
            ;; this one, like "chmod".
            (setq ret      (cons (cons name desc) ret)
                  prev     name
                  prevsect sect
                  prevdesc desc))

          ;; always an entry like "1 chmod" or "2 chmod" with the sect
          (setq ret (cons (cons (concat sect " " name) desc)
                          ret))))

      (setq man-completion-cache (nreverse ret))))))
