;; Copyright 2009, 2010, 2014 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Check that all the :group's specified in defcustom forms refer to known
;; defgroups.
;;
;; defcustom puts itself in a group by adding to the group's custom-group
;; property.  If that group name has a defgroup then it has a
;; group-documentation property too, if not then the defcustom may be a
;; typo.
;;

(defun my-custom-group-exists ()
  (require 'cl)
  (let (lst
        (count 0))
    (mapatoms
     (lambda (symbol)
       (when (get symbol 'custom-group)
         (incf count)
         (unless (get symbol 'group-documentation)
           (push (list symbol (mapcar 'car (get symbol 'custom-group)))
                 lst)))))

    (dolist (elem (sort* lst 'string-lessp
                         :key 'car))
      (apply 'lwarn
             'my-custom-group-exists
             :error
             "`%S' no group-documentation\nfor variables:\n  %S\n"
             elem))

    (message "checked %d customs" count)))
  

(progn
  (unless (featurep 'my-load-all)
    (load (expand-file-name "my-load-all.el")))
  (my-load-all--custom))

(my-custom-group-exists)

(setq debug-on-error t)
(require 'calendar)
(pp (symbol-plist 'auto-composition) (current-buffer))

(symbol-plist 'paragraph-start)





(defcustom foo
  123
  "docstring"
  :type 'integer)

(defun my-custom-has-group ()
  (require 'cl)
  (let ((known (make-hash-table :test 'eq))
        (group-count 0)
        (custom-count 0)
        lst)
    (mapatoms
     (lambda (symbol)
       (when (get symbol 'custom-group)
         (incf group-count)
         (dolist (var (mapcar 'car (get symbol 'custom-group)))
           (puthash var symbol known)))))

    (mapatoms
     (lambda (symbol)
       (when (get symbol 'custom-type)
         (incf custom-count)
         (unless (gethash symbol known)
           (push symbol lst)))))

    (setq lst (sort* lst 'string-lessp :key 'symbol-name))
    (lwarn 'my-custom-has-group
           :error
           "no custom-group for:\n  %s"
           (mapconcat (lambda (symbol)
                        (symbol-name symbol))
                      lst "\n"))

    (message "checked %d groups %d customs" group-count custom-count)))

(my-custom-has-group)
(symbol-file 'word-wrap 'defvar)
(symbol-file 'word-wrap nil)
(symbol-plist 'browse-url-temp-dir)
(pp (symbol-plist 'mailcap-mode) (current-buffer))


