#!/usr/bin/perl -w

# Copyright 2009, 2010 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


# Usage: ./grep-multiline-keywords.pl
#

use 5.010;
use strict;
use warnings;
use POSIX ();
use Perl6::Slurp;
use IO::Uncompress::Gunzip ;
use Iterator::Simple qw(igrep);
use Iterator::Simple::Locate;

use lib::abs '.';
use MyStuff;
use MyUniqByInode;

my $verbose = 0;

my $it = Iterator::Simple::Locate->new (globs => [ '*.el', '*.el.gz' ]);
$it = igrep { -e $_ } $it;
{ my $uniq = MyUniqByInode->new;
  $it = igrep { $uniq->uniq($_) } $it;
}

while (my $filename = $it->next) {
  if ($verbose) { print "look at $filename\n"; }

  my $str = zslurp ($filename) // next;

  # cf lm-copyright-prefix
  next if $str =~ /Ryde/;
  $str =~ /^;; Copyright [0-9].*/m
    or next;

  # doubled spaces -- not in any
  #
  #   $str =~ /^;; Keywords:.*[^ \t\n][ \t]{2,}.*/m
  #     or next;

  # comma without space
  #
  # /usr/share/doc/fetchmail/contrib/fetchmail-mode.el.gz:8:1:
  # ;; Keywords:   fetchmail,config
  # /usr/share/xemacs21/xemacs-packages/lisp/edit-utils/after-save-commands.el.gz:7:1:
  # ;; Keywords: processes,unix
  # /usr/share/xemacs21/xemacs-packages/lisp/zenirc/zenirc-color.el.gz:9:1:
  # ;; Keywords: zenirc,extensions
  #
  #   $str =~ /^;; Keywords:.*,[^ \t\n].*/m
  #     or next;

  # trailing space
  #
  # /usr/share/xemacs21/xemacs-packages/lisp/vm/vm-avirtual.el.gz:7:1:
  # ;; Keywords:    VM, virtual folders
  # /usr/share/xemacs21/xemacs-packages/lisp/vm/vm-summary-faces.el.gz:7:1:
  # ;; Keywords:    VM
  #
  #   $str =~ /;; Keywords: .*?[ \t]$/m
  #     or next;

  #   $str =~ /;; Keywords: .*\n;;[ \t]{2,}[^[:space:]].*/
  #     or next;

  my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $-[0]);
  print "$filename:$line:$col:\n$&\n";
}

sub zslurp {
  my ($filename) = @_;
  my $str;
  if ($filename =~ /\.gz$/) {
    IO::Uncompress::Gunzip::gunzip ($filename => \$str)
        or do {
          print "Cannot gunzip $filename: $IO::Uncompress::Gunzip::GunzipError\n";
          return;
        };

  } else {
    eval { $str = Perl6::Slurp::slurp($filename); 1 }
      or do {
        #         if ($! == POSIX::ENOENT) {
        #        }
        print "Cannot read $filename: $!\n";
        return;
      };
  }
  return $str;
}
exit 0;
