#!/usr/bin/perl -w

# Copyright 2013 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


# Usage: ./grep-provide.pl
#

use 5.010;
use strict;
use warnings;
use File::Basename;
use POSIX ();
use Perl6::Slurp;
use IO::Uncompress::Gunzip ;
use Iterator::Simple qw(igrep);
use Iterator::Simple::Locate;

use lib::abs '.';
use MyStuff;
use MyUniqByInode;

my $verbose = 0;

my $it = Iterator::Simple::Locate->new
  (regexps => [
               # at most one level down
               qr{^/usr/share/emacs/site-lisp/[^/]+(/[^/]+)?\.el$},
              ]);
$it = igrep { -e $_ } $it;
{ my $uniq = MyUniqByInode->new;
  $it = igrep { $uniq->uniq($_) } $it;
}

while (my $filename = $it->next) {
  if ($verbose) { print "look at $filename\n"; }
  $filename =~ m{([^/]+)\.el$} or die;
  my $basename = $1;

  my $str = zslurp($filename) // next;

  # zap comments "; foo"
  $str =~ s/^[ \t]*;.*//mg;

  while ($str =~ /\(provide '([^)]*)\)/mg) {
    my $form = $&;
    my $symbol = $1;
    next if $symbol eq $basename;
    my ($linenum, $colnum) = MyStuff::pos_to_line_and_column ($str, pos($str));
    print "$filename:$linenum:$colnum: wrong provide\n  $form\n";
  }

  if (! ($str =~ /\(provide \'\Q$basename\E\)/)) {
    my ($line, $col) = MyStuff::pos_to_line_and_column ($str, length($str));
    print "$filename:$line:$col: no provide $basename\n";
  }
}

sub zslurp {
  my ($filename) = @_;
  my $str;
  if ($filename =~ /\.gz$/) {
    IO::Uncompress::Gunzip::gunzip ($filename => \$str)
        or do {
          print "Cannot gunzip $filename: $IO::Uncompress::Gunzip::GunzipError\n";
          return;
        };

  } else {
    eval { $str = Perl6::Slurp::slurp($filename); 1 }
      or do {
        #         if ($! == POSIX::ENOENT) {
        #        }
        print "Cannot read $filename: $!\n";
        return;
      };
  }
  return $str;
}
exit 0;
