/* Copyright 2009 Kevin Ryde.

   This file is part of miscbits-el.

   miscbits-el is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   miscbits-el is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with miscbits-el; see the file COPYING.  Failing that, go to
   <http://www.gnu.org/licenses/>.  */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "sqlite3.h"

int
main (void)
{
  sqlite3 *db;
  sqlite3_stmt *st;
  int ret;
  const char *filename = "/tmp/sixteen.sqdb";
  /* const char *filename16 = "\0/\0t\0m\0p\0/\0s\0i\0x\0t\0e\0e\0n\0.\0s\0q\0d\0b\0\0"; */

  unlink (filename);

  ret = sqlite3_open (filename, &db);
  if (ret != SQLITE_OK) {
    printf ("open error %d\n", ret);
    exit (1);
  }

  /*-------------------------------------------------------------------------*/

  ret = sqlite3_prepare_v2 (db, "PRAGMA encoding = \"UTF-16\"", -1, &st, NULL);
  if (ret != SQLITE_OK) {
    printf ("prepare_v2 PRAGMA error %d %s\n", ret, sqlite3_errmsg(db));
    exit (1);
  }

  ret = sqlite3_step (st);
  if (ret != SQLITE_DONE) {
    printf ("step PRAGMA error %d %s\n", ret, sqlite3_errmsg(db));
    exit (1);
  }

  /*-------------------------------------------------------------------------*/

  ret = sqlite3_prepare_v2 (db, "CREATE TABLE foo (\
    key     TEXT     NOT NULL       PRIMARY KEY,\
    value   TEXT     DEFAULT NULL)", -1, &st, NULL);
  if (ret != SQLITE_OK) {
    printf ("prepare_v2 CREATE error %d %s\n", ret, sqlite3_errmsg(db));
    exit (1);
  }

  ret = sqlite3_step (st);
  if (ret != SQLITE_DONE) {
    printf ("step error %d %s\n", ret, sqlite3_errmsg(db));
    exit (1);
  }

  /*-------------------------------------------------------------------------*/

  ret = sqlite3_prepare_v2 (db, "INSERT INTO foo VALUES('k',?)", -1, &st, NULL);
  if (ret != SQLITE_OK) {
    printf ("prepare_v2 INSERT error %d %s\n", ret, sqlite3_errmsg(db));
    exit (1);
  }

  {
    const char str16[] = "h\0e\0l\0l\0o\0 \0w\0o\0r\0l\0d\0";
    ret = sqlite3_bind_text16 (st, 1, str16, sizeof(str16)-1, SQLITE_STATIC);
    if (ret != SQLITE_OK) {
      printf ("bind 1 error %d %s\n", ret, sqlite3_errmsg(db));
      exit (1);
    }
  }
  /* "\x76\0", 2 */
  /*   ret = sqlite3_bind_text16 (st, 2, "\0\x76", 2, SQLITE_STATIC); */
  /*   if (ret != SQLITE_OK) { */
  /*     printf ("bind 2 error %d %s\n", ret, sqlite3_errmsg(db)); */
  /*     exit (1); */
  /*   } */

  ret = sqlite3_step (st);
  if (ret != SQLITE_DONE) {
    printf ("step error %d %s\n", ret, sqlite3_errmsg(db));
    exit (1);
  }

  /*-------------------------------------------------------------------------*/

  printf ("done\n");
  return 0;
}
