;; Copyright 2006, 2007, 2009, 2010, 2013, 2019 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (require 'bytecomp)
    (unless (featurep 'bytecomp-simplify)
      (error "`bytecomp-simplify' not autoloaded"))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; load

    (require 'bytecomp-simplify)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))

    ;; development info ...
    (message "bytecomp-simplify-put--declare-indent-p is %S"
             bytecomp-simplify-put--declare-indent-p)


    ;;-------------------------------------------------------------------------
    ;; handlers are functions

    (mapatoms
     (lambda (symbol)
       (let ((handler (get symbol 'bytecomp-simplify-warn)))
         (when handler
           (unless (or (functionp handler)
                       (and (listp handler)
                            (not (memq nil (mapcar 'functionp handler)))))
             (error "`%S' handler not a function: %S"
                    symbol handler))
           (setq count (1+ count))))))


    ;;-------------------------------------------------------------------------
    ;; byte-compile-warnings customize addition

    (if (not (get 'byte-compile-warnings 'custom-type))
        (message "skip, byte-compile-warnings not a defcustom")

      (let ((type (get 'byte-compile-warnings 'custom-type)))
        (when (eq 'choice (car-safe type))
          (let ((set (assq 'set type)))
            (when set
              (unless (member '(const simplify) set)
                (error "`byte-compile-warnings' not extended:\n%S" type))
              (setq count (1+ count)))))))

    ;;-------------------------------------------------------------------------
    ;; bytecomp-simplify-regexp-fixed-p

    (mapcar (lambda (elem)
              (let* ((want   (nth 0 elem))
                     (regexp (nth 1 elem))
                     (got    (not (not (bytecomp-simplify-regexp-fixed-p regexp)))))
                (unless (equal want got)
                  (error "bytecomp-simplify-regexp-fixed-p: %S want %S got %S"
                         regexp want got))
                (setq count (1+ count))))

            '((nil "[^]]")
              (t   "")
              (t   "\n")
              (t   "\r\n\f")
              (nil "\\\n")
              (t   "\\\\")
              (t   "[a]")
              (nil "[ab]")
              (t   "[\n]")
              (t   "\\*")
              (nil "a*")
              (nil "a?")
              (nil "a+")
              (nil ".")
              (nil "..")
              (nil "\\..")
              (t   "\\.\\.")
              (nil "^")
              (nil "\\`x")
              (t   "abc")
              ))

    ;;-------------------------------------------------------------------------
    ;; bytecomp-simplify-quoted-symbol

    (mapcar (lambda (elem)
              (let* ((want (nth 0 elem))
                     (obj  (nth 1 elem))
                     (got  (bytecomp-simplify-quoted-symbol obj)))
                (unless (equal want got)
                  (error "bytecomp-simplify-quoted-symbol %S want %S got %S"
                         obj want got))
                (setq count (1+ count))))

            '((foo (quote foo))
              (nil (quote nil))
              (nil (quote "x"))
              (nil (quote (quote foo)))
              (nil 123)
              (nil nil)
              (nil "x")
              ))

    ;;-------------------------------------------------------------------------
    ;; byte-compile generating warnings

    (require 'bytecomp)
    (let ((byte-compile-warnings '(simplify))
          (old-warn        (symbol-function 'byte-compile-warn))
          got-warnings) ;; list of strings
      (unwind-protect
          (progn
            (fset 'byte-compile-warn
                  (lambda (fmt &rest args)
                    (setq got-warnings
                          (cons (apply 'format fmt args)
                                got-warnings))))

            (mapcar
             (lambda (feature)
               (when feature
                 (message "with '%S" feature)
                 (require feature))

               (mapcar (lambda (elem)
                         ;; (message "%S" elem)
                         (let ((want-regexp (nth 0 elem))
                               (form        (nth 1 elem))
                               (varsym      (nth 2 elem)))
                           (if (and varsym
                                    (not (symbol-value varsym)))
                               (message "skip, %S is false" varsym)

                             (setq got-warnings nil)
                             (byte-compile-sexp form)

                             (let ((ok (if want-regexp
                                           (and (>= (length got-warnings) 1)
                                                (memq t (mapcar (lambda (str)
                                                                  (not (not (string-match want-regexp str))))
                                                                got-warnings)))
                                         (null got-warnings))))
                               (unless ok
                                 (error "form %S\ngot %S\nwant %S"
                                        form got-warnings want-regexp))
                               (setq count (1+ count))))))

                       `(
                         ;; char-before, char-after, push-mark
                         ("point"  (lambda () (char-before (point))))
                         ("point"  (lambda () (char-after (point))))
                         ("point"  (lambda () (push-mark (point))))

                         ;; delete-windows-on
                         ("(delete-windows-on)"  (lambda () (delete-windows-on (current-buffer)))
                          bytecomp-simplify-delete-windows-on--optarg)

                         ;; (eq nil)
                         ("null" (lambda (obj) (eq nil obj)))
                         ("null" (lambda (obj) (eq obj nil)))
                         (nil    (lambda (x y) (eq x y)))
                         ("eql nil" (lambda (obj) (eql nil obj)) nil t)
                         ("null" (lambda (obj) (equal nil obj)))
                         ("null" (lambda (obj) (equalp nil obj)))
                         ("null" (lambda (obj)
                                   (equal-including-properties nil obj)))
                         ("null" (lambda (obj) (equalp obj nil)))


                         ;; (equal 'symbol)
                         (nil    (lambda (obj) (eq 'foo obj)))
                         ("'symbol" (lambda (obj) (equal 'foo obj)))
                         ("'symbol" (lambda (obj) (equal obj 'foo)))
                         ("'symbol" (lambda (obj) (equalp obj 'foo)))
                         ("'symbol" (lambda (obj) (equal-including-properties obj 'foo)))
                         ("'symbol" (lambda (obj) (eql obj 'foo)))


                         ;; princ
                         ("terpri" (lambda () (princ "\n")))

                         ;; put 'lisp-indent-function
                         ("declare" (progn
                                      (eval-and-compile
                                        (defmacro t-bytecomp-simplify--macro1 ()
                                          '(message "hi")))
                                      (put 't-bytecomp-simplify--macro1
                                           'lisp-indent-function 1))
                          bytecomp-simplify-put--declare-indent-p)
                         ("declare" (progn
                                      (put 't-bytecomp-simplify--macro2
                                           'lisp-indent-function 1)
                                      (eval-and-compile
                                        (defmacro t-bytecomp-simplify--macro2 ()
                                          '(message "hi"))))
                          bytecomp-simplify-put--declare-indent-p)
                         (nil (progn
                                (defun t-bytecomp-simplify--func1 ()
                                  '(message "hi"))
                                (put 't-bytecomp-simplify--func1
                                     'lisp-indent-function 1)))
                         (nil (put 't-bytecomp-simplify--undefined-name
                                   'lisp-indent-function 1))

                         ;; re-search-forward, re-search-backward
                         ("fixed-string" (lambda () (re-search-forward "x")))
                         ("fixed-string" (lambda () (re-search-backward "x")))

                         ;; re-search-forward, re-search-backward,
                         ;; search-forward, search-backward
                         ("point-max" (lambda ()
                                        (re-search-forward "[ab]" (point-max))))
                         ("point-min" (lambda ()
                                        (re-search-backward "[ab]" (point-min))))
                         ("point-max" (lambda ()
                                        (search-forward "x" (point-max))))
                         ("point-min" (lambda ()
                                        (search-backward "x" (point-min))))
                         ;; opposites
                         (nil (lambda () (re-search-forward "[ab]" (point-min))))
                         (nil (lambda () (re-search-backward "[ab]" (point-max))))
                         (nil (lambda () (search-forward "x" (point-min))))
                         (nil (lambda () (search-backward "x" (point-max))))

                         ;; up-list, down-list
                         ("(up-list)"   (lambda () (up-list 1))
                          bytecomp-simplify-updown-list--optarg)
                         (nil           (lambda () (up-list -1)))
                         (nil           (lambda () (up-list 2)))
                         ("(down-list)" (lambda () (down-list 1))
                          bytecomp-simplify-updown-list--optarg)
                         (nil           (lambda () (down-list 2)))
                         (nil           (lambda () (down-list -1)))
                         ))


               (let ((filename "bytecomp-simplify--indent.el"))
                 (if (not bytecomp-simplify-put--declare-indent-p)
                     (message "skip %S in this emacs" filename)

                   (setq got-warnings nil)
                   (byte-compile-file filename)
                   (condition-case nil
                       (delete-file (concat filename "c"))
                     (error nil))

                   (let ((error-count 0))
                     (mapcar (lambda (macroname)
                               (let (found)
                                 (mapcar (lambda (warning)
                                           (if (string-match macroname warning)
                                               (setq found t)))
                                         got-warnings)
                                 (unless found
                                   (message "no warning for %S"
                                            macroname)
                                   (setq error-count (1+ error-count)))))
                             '("t-bytecomp-simplify--macro1"
                               "t-bytecomp-simplify--macro2"
                               "t-bytecomp-simplify--macro3"
                               "t-bytecomp-simplify--macro4"
                               "t-bytecomp-simplify--macro5"
                               "t-bytecomp-simplify--macro6"))
                     (unless (= 0 error-count)
                       (error "got %S" got-warnings)))

                   (setq count (1+ count)))))

             '(nil cl)))

        (fset 'byte-compile-warn old-warn)))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'bytecomp-simplify))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'bytecomp-simplify))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))
