;; Copyright 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (case-fold-search nil))

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(compilation-always-kill-mode))


    ;;-------------------------------------------------------------------------
    ;; load

    (require 'compilation-always-kill)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- defadvice cleanup, emacs22 up

    (require 'compilation-always-kill)

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'compilation-always-kill))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- yes-or-no-p still works

    (require 'advice)
    (if (ad-is-advised 'yes-or-no-p)
        (ad-deactivate 'yes-or-no-p))
    (let ((ad-redefinition-action 'accept)
          (orig (symbol-function 'yes-or-no-p))
          (want 1234))
      (fset 'yes-or-no-p (lambda (prompt) want))
      (if (ad-is-advised 'yes-or-no-p)
          (ad-activate 'yes-or-no-p))

      (require 'compilation-always-kill)
      (unload-feature 'compilation-always-kill t)

      (let ((got (yes-or-no-p "test run yes-or-no-p ... ")))
        (unless (equal got want)
          (error "yes-or-no-p not quite right, maybe")))

      (if (ad-is-advised 'yes-or-no-p)
          (ad-deactivate 'yes-or-no-p))
      (fset 'yes-or-no-p orig)
      (if (ad-is-advised 'yes-or-no-p)
          (ad-activate 'yes-or-no-p))

      (setq count (1+ count)))


    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'compilation-always-kill))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))

