;; Copyright 2006, 2007, 2009, 2010, 2011, 2013 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(unless (fboundp 'file-expand-wildcards)
  (load (expand-file-name "file-expand-wildcards.el")))

;; (cond ((fboundp 'make-temp-file)
;;        (defalias 'my-make-temp-file 'make-temp-file))
;; 
;;       ((condition-case nil
;;            (require 'mm-util)
;;          (error nil))
;;        (defalias 'my-make-temp-file 'mm-make-temp-file)))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-do-not-edit--temp*"))
    (switch-to-buffer tempbuf)
    (emacs-lisp-mode)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(do-not-edit-readonly
              do-not-edit-perl-blib))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            (if (boundp 'find-file-hook)
                ;; emacs22
                '((find-file-hook do-not-edit-perl-blib)
                  (find-file-hook do-not-edit-readonly))
              ;; xemacs21
              '((find-file-hooks  do-not-edit-perl-blib)
                (find-file-hooks  do-not-edit-readonly))))

    ;;-------------------------------------------------------------------------
    ;; do-not-edit-readonly - package-user-dir

    (find-file "~/.emacs.d/elpa/foo-1/foo-autoloads.el")
    (setq buffer-read-only nil)
    (when buffer-read-only
      (error "oops, visit is already readonly %S"
             (buffer-file-name)))
    (do-not-edit-readonly)
    (when buffer-read-only
      (error "do-not-edit-readonly should not apply to elpa foo-autoloads.el"))
    (setq count (1+ count))
    (kill-buffer (current-buffer))

    ;;-------------------------------------------------------------------------
    ;; do-not-edit-readonly - perl Devel::PPPort

    (shell-command "perl -MDevel::PPPort -e 'Devel::PPPort::WriteFile(\"/tmp/t-do-not-edit--ppport.h\")'")
    (when (condition-case err
              (progn
                (erase-buffer)
                (insert-file-contents "/tmp/t-do-not-edit--ppport.h")
                (delete-file "/tmp/t-do-not-edit--ppport.h")
                t)
            (error
             (message "skip, cannot make ppport.h: %S"
                      (error-message-string err))
             nil))

      (setq buffer-read-only nil)
      (do-not-edit-readonly)
      (unless buffer-read-only
        (error "do-not-edit-readonly pport.h didn't get readonly"
               got want))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; do-not-edit-readonly - some files

    (auto-compression-mode 1)

    (let ((case-fold-search t))
      (mapcar (lambda (elem)
                (let* ((want      (nth 0 elem))
                       (wildcard  (nth 1 elem))
                       (file-list (file-expand-wildcards wildcard)))
                  (if (null file-list)
                      (message "skip, no files: %s" wildcard)

                    (mapcar (lambda (filename)
                              (find-file filename)
                              (setq buffer-read-only nil)
                              (do-not-edit-readonly)
                              (let ((got buffer-read-only))
                                (unless (equal got want)
                                  (error "do-not-edit-readonly %S got %S want %S"
                                         filename got want))
                                (setq count (1+ count)))

                              (kill-buffer nil))
                            file-list))))

              `(
                (t   "/usr/lib/perl/5.10.1/auto/DynaLoader/*.al")
                (nil "/usr/share/perl/*/AutoSplit.pm")

                ;; `autoload-rubric'
                (t   "/usr/share/emacs/*/lisp/loaddefs.el")

                ;; plain DO NOT MODIFY
                (t "/usr/share/xemacs-*/lisp/auto-autoloads.el.gz"))))


    ;;-------------------------------------------------------------------------
    ;; do-not-edit-perl-blib

    (let ((case-fold-search t))
      (mapcar (lambda (elem)
                (let* ((want     (nth 0 elem))
                       (filename (nth 1 elem)))
                  (message "do-not-edit-perl-blib on %S" filename)
                  (setq buffer-file-name filename)
                  (setq buffer-read-only nil)
                
                  (let ((got (condition-case err
                                 (progn
                                   (do-not-edit-perl-blib)
                                   buffer-read-only)
                               (error
                                (list 'error (error-message-string err))))))
                    (unless (equal got want)
                      (error "do-not-edit-perl-blib %S\ngot  %S\nwant %S"
                             filename got want))
                    (setq count (1+ count)))))

              `((nil "/etc/motd")
                (nil "/etc/motd/filename/used/as/if/directory")
                (nil "/etc/blibbing/motd")
                (nil "/etc/ablib/motd")

                (t   "/tmp/foo/blib/lib/Foo.pm")
                (nil "/tmp/BLIB/foo.pl")
                )))
    (setq buffer-file-name nil)


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    ;; couple of cycles
    (require 'do-not-edit)
    (unload-feature 'do-not-edit t)
    (require 'do-not-edit)
    (unload-feature 'do-not-edit t)
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
