;; Copyright 2007, 2009, 2010 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-nroff-filladapt--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(nroff-filladapt-setups))

    ;;-------------------------------------------------------------------------

    (nroff-filladapt-setups)

    (mapcar (lambda (elem)
              (let ((want (car elem))
                    (str  (cadr elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))
                (let ((got (looking-at paragraph-separate)))
                  (or (eq want got)
                      (error "paragraph-separate got %s want %s" got want)))
                (setq count (1+ count))))
            '((t   "\\\"\n")  ;; \"
              (t   "\\\" \n") ;; \" space
              (t   "\\\#\n")  ;; \#
              (t   "\\\# \n") ;; \# space

              (t   ".\n")     ;; .
              (t   "'\n")     ;; '
              (t   ". \n")    ;; . space
              (t   "' \n")    ;; ' space

              (t   ".P\n")    ;; .P
              (t   "'P\n")    ;; 'P
              (t   ". P\n")   ;; . P
              (t   "' P\n")   ;; ' P
              (t   ".TH\n")   ;; .TH

              (t   ".\\\n")   ;; .\
              (t   "'\\\n")   ;; '\
              (t   ". \\\n")  ;; . \
              (t   "' \\\n")  ;; ' \

              (t   ".\\X\n")   ;; .\X
              (t   ". \\X\n")  ;; . \X
              (t   "'\\X\n")   ;; '\X
              (t   "' \\X\n")  ;; ' \X

              (t   ".\\\"\n")
              (t   ".\\\" \n")
              (t   ".\\\#\n")
              (t   ".\\\# \n")

              (t   ". \\\"\n")
              (t   ". \\\" \n")
              (t   ". \\\#\n")
              (t   ". \\\# \n")

              (nil "\\\" hello\n")    ;; \" hello
              (nil "\\# hello\n")     ;; \# hello

              (nil ".\\\" hello\n")   ;; .\" hello
              (nil ".\\# hello\n")    ;; .\# hello
              (nil "'\\\" hello\n")   ;; '\" hello
              (nil "'\\# hello\n")    ;; '\# hello

              (nil ". \\\" hello\n")   ;; .\" hello
              (nil ". \\# hello\n")    ;; .\# hello
              (nil "' \\\" hello\n")   ;; '\" hello
              (nil "' \\# hello\n")    ;; '\# hello

              (t   "'''\n")         ;; '''
              (t   "''' \n")        ;; ''' space
              (nil "''' hello\n")   ;; ''' hello

              ))

    (mapcar (lambda (elem)
              (let ((want (car elem))
                    (str  (cadr elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))
                (looking-at comment-start-skip)
                (let ((got (- (match-end 0) (point-min))))
                  (or (= want got)
                      (error "comment-start-skip got %s want %s" got want)))
                (setq count (1+ count))))
            '((2   "\\\"\n") ;;     ;; \"
              (3   "\\\" \n")       ;; \" space
              (3   "\\\" hello\n")  ;; \" hello

              (2   "\\#\n")        ;; \#
              (3   "\\# \n")       ;; \# space
              (3   "\\# hello\n")  ;; \# hello

              (3   ".\\\"\n")       ;; .\"
              (4   ".\\\" \n")      ;; .\" space
              (4   ".\\\" hello\n") ;; .\" hello
              (3   ".\\#\n")       ;; .\#
              (4   ".\\# \n")      ;; .\# space
              (4   ".\\# hello\n") ;; .\# hello

              (4   ". \\\"\n")
              (5   ". \\\" \n")
              (5   ". \\\" hello\n")
              (4   ". \\#\n")
              (5   ". \\# \n")
              (5   ". \\# hello\n")

              (4   "' \\\"\n")
              (5   "' \\\" \n")
              (5   "' \\\" hello\n")
              (4   "' \\#\n")
              (5   "' \\# \n")
              (5   "' \\# hello\n")

              (3   "'''\n")         ;; '''
              (4   "''' \n")        ;; ''' space
              (4   "''' hello\n")   ;; ''' hello

              ))

    (mapcar (lambda (elem)
              (let ((want (car elem))
                    (str  (cadr elem)))
                (erase-buffer)
                (insert str)
                (goto-char (point-min))
                (let ((got (looking-at auto-fill-inhibit-regexp)))
                  (or (eq want got)
                      (error "auto-fill-inhibit-regexp got %s want %s"
                             got want))))
              (setq count (1+ count)))
            '((nil "\\\"\n") ;;     ;; \"
              (nil "\\\" \n")       ;; \" space
              (nil "\\\" hello\n")  ;; \" hello

              (nil "\\#\n")        ;; \#
              (nil "\\# \n")       ;; \# space
              (nil "\\# hello\n")  ;; \# hello

              (nil ".\\\"\n")       ;; .\"
              (nil ".\\\" \n")      ;; .\" space
              (nil ".\\\" hello\n") ;; .\" hello
              (nil ".\\#\n")       ;; .\#
              (nil ".\\# \n")      ;; .\# space
              (nil ".\\# hello\n") ;; .\# hello

              (nil ". \\\"\n")
              (nil ". \\\" \n")
              (nil ". \\\" hello\n")
              (nil ". \\#\n")
              (nil ". \\# \n")
              (nil ". \\# hello\n")

              (nil "' \\\"\n")
              (nil "' \\\" \n")
              (nil "' \\\" hello\n")
              (nil "' \\#\n")
              (nil "' \\# \n")
              (nil "' \\# hello\n")

              (nil "'''\n")         ;; '''
              (nil "''' \n")        ;; ''' space
              (nil "''' hello\n")   ;; ''' hello

              (t   ".B\n")
              (t   ".TH\n")
              (t   ".SH my heading\n")
              (t   ".\\\n")
              (t   ".\\H my heading\n")
              (t   ". B\n")
              (t   ". TH\n")
              (t   ". SH my heading\n")
              (t   ". \\\n")
              (t   ". \\H my heading\n")

              ))

    (bury-buffer)
    (message "  %d tests" count)
    count))
